"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePropertyProxy = void 0;
const util_1 = require("./util");
const validates_1 = require("./validates");
function usePropertyProxy() {
    let schemaObj;
    let schemaTarget;
    function Validate(schama) {
        return function (target) {
            return new Proxy(target, {
                construct: (_target, args) => {
                    schemaTarget = new target(...args);
                    if (schama) {
                        schemaObj = schama;
                        (0, validates_1.zodValidate)(schemaTarget, schama);
                        (0, validates_1.yupValidate)(schemaTarget, schama);
                    }
                    (0, validates_1.classValidate)(schemaTarget);
                    return schemaTarget;
                }
            });
        };
    }
    function Property() {
        return function (target, key) {
            Object.defineProperty(target, key, {
                get: () => "",
                set: function (v) {
                    let val;
                    Object.defineProperty(this, key, {
                        get: () => { var _a, _b; return ((_b = (_a = ObjectProxy[key]) === null || _a === void 0 ? void 0 : _a.getter) === null || _b === void 0 ? void 0 : _b.call(_a, val)) || val; },
                        set: (newValue) => {
                            var _a, _b, _c;
                            if ((_a = ObjectProxy[key]) === null || _a === void 0 ? void 0 : _a.setter) {
                                (_c = (_b = ObjectProxy[key]).setter) === null || _c === void 0 ? void 0 : _c.call(_b, { setValue(value) {
                                        val = value;
                                    }, value: newValue });
                                return;
                            }
                            val = newValue;
                            if (schemaTarget && schemaTarget[key] !== newValue) {
                                Object.assign(schemaTarget, { [key]: newValue });
                            }
                            (0, validates_1.classValidate)(schemaTarget);
                            (0, validates_1.zodValidate)(schemaTarget, schemaObj);
                            (0, validates_1.yupValidate)(schemaTarget, schemaObj);
                        },
                        enumerable: true,
                    });
                    this[key] = v;
                },
            });
        };
    }
    const ObjectProxy = {};
    const proxy = new Proxy({}, {
        get(_target, p) {
            const isSet = p.includes("set");
            const originalParam = isSet ? (0, util_1.lowCaseFirstLetter)(p.split("set")[1]) : (0, util_1.lowCaseFirstLetter)(p.split("get")[1]);
            const obj = {
                [p]: (cb) => {
                    Object.assign(ObjectProxy, {
                        [originalParam]: {
                            [`${isSet ? "setter" : "getter"}`]: cb
                        }
                    });
                }
            };
            return obj[p];
        },
    });
    return { Property, proxy, Validate };
}
exports.usePropertyProxy = usePropertyProxy;
