"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.yupValidate = exports.zodValidate = exports.classValidate = void 0;
const class_validator_1 = require("class-validator");
const zod_1 = require("zod");
const ValidationError_1 = require("./Error/ValidationError");
const yup_1 = require("yup");
const classValidate = (target) => {
    if (target) {
        const errors = (0, class_validator_1.validateSync)(target);
        if (errors.length > 0) {
            const errorsMaped = errors.map(error => {
                return {
                    property: error.property,
                    message: (error.constraints && Object.values(error.constraints)[0]),
                    type: error.constraints && Object.keys(error.constraints)[0]
                };
            });
            throw new ValidationError_1.ValidationError(errorsMaped);
        }
    }
};
exports.classValidate = classValidate;
const zodValidate = (target, schema) => {
    if (schema && target && schema instanceof zod_1.ZodObject) {
        try {
            schema.parse(target);
        }
        catch (_error) {
            const error = _error;
            const errorsMaped = error.errors.map(error => {
                return {
                    property: error.path[0].toString(),
                    message: error.message,
                    type: error.code
                };
            });
            throw new ValidationError_1.ValidationError(errorsMaped);
        }
    }
};
exports.zodValidate = zodValidate;
const yupValidate = (target, schema) => {
    if (schema && target && schema instanceof yup_1.ObjectSchema) {
        schema.validate(target, { abortEarly: false }).catch(_error => {
            const error = _error;
            const errorsMaped = error.inner.map(error => {
                return {
                    property: error.path,
                    message: error.errors[0],
                    type: error.type
                };
            });
            throw new ValidationError_1.ValidationError(errorsMaped);
        });
    }
};
exports.yupValidate = yupValidate;
